""" The exception raised when an unknown extension is referenced. """


class UnknownExtension(Exception):
    """ The exception raised when an unknown extension is referenced. """


#### EOF ######################################################################
